from abc import abstractmethod
from datetime import datetime
from typing import Protocol

from clifinance.domain.expense.model import Expense, ExpenseType


class Session(Protocol):
    @abstractmethod
    def commit(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def rollback(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def add(self, expense: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def update(self, expense: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def delete(self, expense: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def get(
        self,
        expense_id: int | None,
        type: ExpenseType | None,
        date: datetime | None,
        amount: int | None,
    ) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def get_all(self) -> list[Expense]:
        raise NotImplementedError
