from abc import abstractmethod
from datetime import datetime
from typing import Protocol

from clifinance.domain.expense.model import Expense, ExpenseType


class ExpenseGateway(Protocol):
    @abstractmethod
    def get_all_expenses(self) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def add_expense(self, exp: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def update_expense(self, expense: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_type(self, type: ExpenseType) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_date(self, date: datetime) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_price(self, price: float) -> list[Expense]:
        raise NotImplementedError
