from dataclasses import dataclass
from datetime import datetime
from enum import Enum

from clifinance.domain.entity import DomainEntity, DomainValueObject


@dataclass(frozen=True)
class ExpenseId(DomainValueObject):
    value: int


class ExpenseType(Enum):
    INCOME = "Income"
    EXPENSE = "Expense"


@dataclass(frozen=True)
class ExpenseDate(DomainValueObject):
    value: datetime

    def isoformat(self):
        return self.value.isoformat()

    @staticmethod
    def from_isoformat(date: str) -> "ExpenseDate":
        return ExpenseDate(datetime.fromisoformat(date))


@dataclass(frozen=True)
class ExpenseAmount(DomainValueObject):
    value: float


@dataclass(frozen=True)
class ExpenseDescription(DomainValueObject):
    value: str


@dataclass
class Expense(DomainEntity[ExpenseId]):
    type: ExpenseType
    date: ExpenseDate
    amount: ExpenseAmount
    description: ExpenseDescription

    @staticmethod
    def create(id: int, type: str, date: datetime, amount: float, description: str) -> "Expense":
        return Expense(
            id=ExpenseId(id),
            type=ExpenseType(type.title()),
            date=ExpenseDate(date),
            amount=ExpenseAmount(amount),
            description=ExpenseDescription(description),
        )
