import json

from clifinance.domain.expense.model import Expense
from clifinance.infrastructure.persistence.mappers.expense import (
    expense_from_json,
    expense_to_json,
)


class JsonFileDriver:
    def read(self) -> list[Expense]:
        try:
            with open("./app_data.json") as f:
                data = json.load(f)

            return self.__data_from_json(data)
        except FileNotFoundError:
            return list()
        except json.JSONDecodeError:
            return list()

    def write(self, data: list[Expense]) -> None:
        with open("./app_data.json", "w") as f:
            json.dump(self.__data_to_json(data), f)

    def __data_to_json(self, data: list[Expense]) -> list[dict]:
        return [expense_to_json(x) for x in data]

    def __data_from_json(self, data: list[dict]) -> list[Expense]:
        return [expense_from_json(x) for x in data]
