from datetime import datetime

from clifinance.application.abstractions.protocols.session import Session
from clifinance.domain.expense.gateway import ExpenseGateway
from clifinance.domain.expense.model import Expense, ExpenseType


class JsonExpenseGateway(ExpenseGateway):
    def __init__(self, session: Session) -> None:
        self.session = session

    def get_all_expenses(self) -> list[Expense]:
        return self.session.get_all()

    def add_expense(self, exp: Expense) -> None:
        self.session.add(exp)
        self.session.commit()

    def update_expense(self, expense: Expense) -> None:
        self.session.update(expense)

    def get_all_expenses_by_type(self, type: ExpenseType) -> list[Expense]:
        return [x for x in self.session.get_all() if x.type == type]

    def get_all_expenses_by_date(self, date: datetime) -> list[Expense]:
        return [x for x in self.session.get_all() if x.date == date]

    def get_all_expenses_by_price(self, price: float) -> list[Expense]:
        return [x for x in self.session.get_all() if x.amount == price]
