import argparse
import datetime

from clifinance.presentation.cli.commands import (
    add_expense,
    get_balance,
    select_expenses,
)


def get_balance_parser(
    subparser: argparse._SubParsersAction,
) -> argparse.ArgumentParser:
    balance_parser = subparser.add_parser(name="balance", help="Get balance")

    return balance_parser


def get_select_parser(subparser: argparse._SubParsersAction) -> argparse.ArgumentParser:
    select_parser = subparser.add_parser(name="select", help="Get expenses by filter")
    select_parser.add_argument(
        "-d",
        "--date",
        type=datetime.date.fromisoformat,
        help="Select expenses by date",
    )
    select_parser.add_argument(
        "-p",
        "--price",
        type=float,
        help="Select expenses by price",
    )
    select_parser.add_argument(
        "-t",
        "--type",
        type=str,
        help="Select expenses by type. Available values: income, expense",
    )
    return select_parser


def get_add_parser(subparser: argparse._SubParsersAction) -> argparse.ArgumentParser:
    add_parser = subparser.add_parser(name="add", help="Add new expense")
    add_parser.add_argument(
        "-t",
        "--type",
        type=str,
        help="Expense type. Available values: income, expense",
    )
    add_parser.add_argument(
        "-d",
        "--date",
        type=datetime.datetime.fromisoformat,
        help="Expense date",
    )
    add_parser.add_argument(
        "-a",
        "--amount",
        type=float,
        help="Expense price",
    )
    add_parser.add_argument(
        "-D",
        "--description",
        type=str,
        help="Expense description",
    )
    return add_parser


def provide_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(description="Cli finance app")
    subparser = parser.add_subparsers(title="Availeble commands", dest="command")

    balance_parser = get_balance_parser(subparser=subparser)
    balance_parser.set_defaults(func=get_balance)

    select_parser = get_select_parser(subparser=subparser)
    select_parser.set_defaults(func=select_expenses)

    add_parser = get_add_parser(subparser=subparser)
    add_parser.set_defaults(func=add_expense)
    return parser
